﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjektStatki
{
    public class Camera
    {
        // Position of camera
        private int _xView;
        private int _yView;
        public int xView { get => _xView; }
        public int yView { get => _yView; }

        // viewport dimensions
        private int _wView;
        private int _hView;
        public int wView { get => _wView; }
        public int hView { get => _hView; }

        // distance from followed object to border before camera starts move
        private int xDeadZone;
        private int yDeadZone;

        // followed object
        private GameObject target;

        // Rectangle with world size
        private Rectangle worldRectangle;

        // Rectangle reprezenting this viewport
        private Rectangle viewportRectangle;

        public Camera(int xView, int yView, int canvasWidth, int canvasHeight, int worldWidth, int worldHeight)
        {
            this._xView = xView;
            this._yView = yView;
            this._wView = canvasWidth;
            this._hView = canvasHeight;
            this.viewportRectangle = new Rectangle(this.xView, this.yView, this.wView, this.hView);
            this.worldRectangle = new Rectangle(0, 0, worldWidth, worldHeight);
        }

        public void Update()
        {
            // keep following object
            if (this.target != null)
            {
                // moves camera on horizontal
                if (this.target.x - this._xView + this.xDeadZone > this._wView)
                    this._xView = this.target.x - (this._wView - this.xDeadZone);
                else if (this.target.x - this.xDeadZone < this._xView)
                    this._xView = this.target.x - this.xDeadZone;

                // moves camera vertical
                if (this.target.y - this._yView + this.yDeadZone > this._hView)
                    this._yView = this.target.y - (this._hView - this.yDeadZone);
                else if (this.target.y - this.yDeadZone < this.yView)
                    this._yView = this.target.y - this.yDeadZone;
            }

            // update viewportRect
            this.viewportRectangle.Set(this.xView, this.yView);

            // don't let camera leaves the world's boundary
            if (!this.viewportRectangle.Within(this.worldRectangle))
            {
                if (this.viewportRectangle.left < this.worldRectangle.left) { this._xView = this.worldRectangle.left; }
                if (this.viewportRectangle.top < this.worldRectangle.top) { this._yView = this.worldRectangle.top; }
                if (this.viewportRectangle.right > this.worldRectangle.right) { this._xView = this.worldRectangle.right - this.wView; }
                if (this.viewportRectangle.bottom > this.worldRectangle.bottom) { this._yView = this.worldRectangle.bottom - this.hView; }

            }
        }

        public void AttachToObject(GameObject gameObject, int xDeadZone = 0, int yDeadZone = 0)
        {
            this.target = gameObject;
            this.xDeadZone = xDeadZone;
            this.yDeadZone = yDeadZone;
        }

        public void ZoomIn()
        {

        }

        public void ZoomOut()
        {

        }
    }
}
