﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace ProjektStatki
{
    public class Enemy_1 : Enemy
    {
        public Enemy_1()
        {
            this.bitmap = Game1.enemy1Image;
            this.bitmapData = new Color[this.bitmap.Width * this.bitmap.Height];
            this.bitmap.GetData<Color>(this.bitmapData);
            this.xPos = 280;
            this.yPos = 40;
            this.hitbox = new Rectangle((int)xPos, (int)yPos, this.bitmap.Width, this.bitmap.Height);
            this.Health = 100;
            this.speed = 180;

            this.cannons = new List<Cannon>();
            this.AddCannon(new BasicCannon(this));

            // Subscribe self to game objects
            Game1.gObjects.Add(this);
            Game1.ships.Add(this);
        }

        public override void Draw(SpriteBatch spriteBatch, Camera C)
        {
            spriteBatch.Begin();
            spriteBatch.Draw(this.bitmap, new Vector2(this.x  + this.width/2 - C.xView, this.y + this.height/2 - C.yView), null, Color.White, this.rotation, new Vector2(this.width / 2, this.height / 2), 1, SpriteEffects.None, 0);
            spriteBatch.End();
        }

        public override void Update()
        {
            // check if alive
            if (this.health <= 0) {
                Game1.DeleteObject(this);
            } 

            // update tile position
            if (Math.Ceiling((double)this.x/64) != this.xTile || Math.Ceiling((double)this.y/64) != this.yTile) {
                Debug.WriteLine("Change path");
                this.xTile = (int)Math.Ceiling((double)this.x / 64);
                this.yTile = (int)Math.Ceiling((double)this.y / 64);
                Navigate();
                GetNextLocation();
            }
            
            // navigate to player
            this.direction = new Vector2(nextLocation.X - currentLocation.X, nextLocation.Y - currentLocation.Y);     
            this.direction.Normalize();
            xVelocity = direction.X;
            yVelocity = direction.Y;
            double playerDistance = Math.Sqrt(Math.Pow(Math.Abs(x - Game1.Player.x), 2) + Math.Pow(Math.Abs(y - Game1.Player.y), 2));
            if ((xVelocity != 0 || yVelocity != 0) && playerDistance > 300) { this.Move(); }

            this.HandleCannnon();

            // check objects collisions
            //this.HandleObjectsCollision();

            // don't let character leave the world
            if (!this.hitbox.Within(Game1.worldRectangle))
            {
                if (this.x  < 0) { this.xPos = this.width / 2; }
                if (this.y  < 0) { this.yPos = this.height / 2; }
                if (this.x  > Game1.worldWidth) { this.xPos = Game1.worldWidth - this.width / 2; }
                if (this.y  > Game1.worldHeight) { this.yPos = Game1.worldHeight - this.height / 2; }
                this.hitbox.Set((int)xPos, (int)yPos);
            }
        }

        private void Move()
        {
            if (!Double.IsNaN(xVelocity)) { this.xPos += xVelocity * Game1.STEP * this.speed; }
            if (!Double.IsNaN(yVelocity)) { this.yPos += yVelocity * Game1.STEP * this.speed; }
            this.rotation = Game1.VectorToAngle(this.direction) + MathHelper.Pi;

            this.hitbox.Set((int)xPos, (int)yPos);
        }

        private void MoveBack()
        {
            if (!Double.IsNaN(xVelocity)) { this.xPos -= xVelocity * Game1.STEP * this.speed; }
            if (!Double.IsNaN(yVelocity)) { this.yPos -= yVelocity * Game1.STEP * this.speed; }
            this.rotation = Game1.VectorToAngle(this.direction) + MathHelper.Pi;

            this.hitbox.Set((int)xPos, (int)yPos);
        }

        protected override void HandleObjectsCollision()
        {
            foreach (Map.Chunk tile in Game1.Level.Tiles)
            {
                int tileDist = Math.Abs(this.x / 64 - tile.col);
                tileDist += Math.Abs(this.y / 64 - tile.row);
                if (tile.collidable && tileDist < 2 && this.Overlaps(tile))
                {
                    this.MoveBack();
                }
            }
        }

        protected override void HandleCannnon()
        {
            if (this.activeCannon.cooldown > 0)
            {
                this.activeCannon.cooldown -= 1;
            }
        }

        private void AddCannon(Cannon cannon)
        {
            this.cannons.Add(cannon);
            this.activeCannon = cannon;
        }
    }
}
