﻿using MapExtension;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Xml;

namespace ProjektStatki
{
    /// <summary>
    /// This is the main type for your game.
    /// </summary>
    public class Game1 : Game
    {
        public const int FPS = 60;
        public const double INTERVAL = 1000 / FPS;  // milliseconds between one frame
        public const double STEP = INTERVAL / 1000; // seconds between one frame
        public static Rectangle worldRectangle = new Rectangle(0, 0, 3200, 3200);
        public const int worldWidth = 3200;
        public const int worldHeight = 3200;
        public const int tilesCount = 97; // 96 different tiles

        public static List<GameObject> gObjects = new List<GameObject>();
        public static List<Character> ships = new List<Character>();
        public static Vector2 inputMove;

        static Map level;
        public static Map Level { get => level; }
        static Player player;
        public static Player Player { get => player; }
        static Camera mainCamera;


        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Vector2 position;
        public static Texture2D canonBallImage, playerImage, enemy1Image;
        TiledMapData mapData;
        Texture2D mapImage;
        public static Texture2D[] tileImages = new Texture2D[tilesCount];
        // UI textures
        public static Texture2D helthBar;
        public static Texture2D healthBarWrapper;
        public static Texture2D[] activeCanons = new Texture2D[5];
        public static Texture2D[] inactiveCanons = new Texture2D[5];

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1400;  // set this value to the desired width of your window
            graphics.PreferredBackBufferHeight = 800;   // set this value to the desired height of your window
            graphics.ApplyChanges();
            //graphics.IsFullScreen = true;
            Content.RootDirectory = "Content";
        }
        
        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            position = new Vector2(0, 0);
            inputMove = new Vector2(0, 0);
            IsMouseVisible = true;

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);


            // TODO: use this.Content to load your game content here
            /* TEXTURES && FILES */
            mapData = Content.Load<TiledMapData>("map1");
            mapImage = Content.Load<Texture2D>("graphics/map_image");
            canonBallImage = Content.Load<Texture2D>("graphics/cannon_ball");
            playerImage = Content.Load<Texture2D>("graphics/pirate_ship");
            enemy1Image = Content.Load<Texture2D>("graphics/yellow_ship");
            for (int i = 1; i < tilesCount; i++) {
                tileImages[i] = Content.Load<Texture2D>("tiles/tile_" + i.ToString());
            }

            /* GRAPHICS DEPENDANT OBJECTS */
            level = new Map(mapImage, mapData);
            mainCamera = new Camera(0, 0, this.GraphicsDevice.Viewport.Width, this.GraphicsDevice.Viewport.Height, level.width, level.height);
            player = new Player();
            mainCamera.AttachToObject(player, this.GraphicsDevice.Viewport.Width / 2, this.GraphicsDevice.Viewport.Height / 2);
            new Enemy_1();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// game-specific content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // TODO: Add your update logic here
            // Inputs
            // Get current state of mouse
            MouseState mState = Mouse.GetState();
            position.X = mState.X;
            position.Y = mState.Y;

            // Handle shooting with right button
            if (mState.RightButton == ButtonState.Pressed)
            {
                Vector2 direction = new Vector2(mState.X + mainCamera.xView - player.x, mState.Y + mainCamera.yView - player.y );
                direction.Normalize();
                player.ActiveCannon.Shoot(direction);
            }

            // Handle move with left button
            if (mState.LeftButton == ButtonState.Pressed)
            {
                inputMove = new Vector2(mState.X + mainCamera.xView - player.x, mState.Y + mainCamera.yView - player.y);
                inputMove.Normalize();
            }
            else
            {
                inputMove = new Vector2(0, 0);
            }

            KeyboardState kState = Keyboard.GetState();

            // Game objects
            for (int i = 0; i < gObjects.Count; i++)
            {
                gObjects[i].Update();
            }
            mainCamera.Update();

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            for (int i = 0; i < gObjects.Count; i++)
            {
                gObjects[i].Draw(spriteBatch, mainCamera);
            }

            base.Draw(gameTime);
        }

        public static void DeleteObject(GameObject obj)
        {
            gObjects.Remove(obj);
            if (obj is Character) { ships.Remove((Character)obj); }
        }

        public static Vector2 AngleToVector(float angle)
        {
            return new Vector2((float)Math.Sin(angle), -(float)Math.Cos(angle));
        }

        public static float VectorToAngle(Vector2 vector)
        {
            return (float)Math.Atan2(vector.X, -vector.Y);
        }
    }
}
